@@ -1,4 +1,4 @@
-// $Id: ConfigDefaults.cc,v 1.1 2000/08/30 22:55:28 dave Exp $
+// $Id: ConfigDefaults.cc,v 1.2 2002/10/01 18:04:31 flaterco Exp $
 /*  ConfigDefaults  Defaults initialized to settings in config.hh.
 
     Copyright (C) 1998  David Flater.
@@ -28,6 +28,7 @@ ConfigDefaults::ConfigDefaults () {
   colors[Colors::datum] = datumdefcolor;
   colors[Colors::ebb] = ebbdefcolor;
   el = extralines;
+  fe = flatearth;
   colors[Colors::flood] = flooddefcolor;
   colors[Colors::mark] = markdefcolor;
   colors[Colors::msl] = msldefcolor;
@@ -48,6 +49,7 @@ ConfigDefaults::ConfigDefaults () {
   tl = toplines;
   nf = nofill;
   ns = nosunmoon;
+  in = infer;
   assert (deflwidth > 0.0);
   lw = deflwidth;
   gl = defgl;
