@@ -1,4 +1,4 @@
-// $Id: ConstantSetWrapper.cc,v 1.1 2001/03/25 19:50:25 dave Exp $
+// $Id: ConstantSetWrapper.cc,v 1.3 2002/12/19 18:40:45 flaterco Exp $
 /*  ConstantSetWrapper
 
     Copyright (C) 1998  David Flater.
@@ -102,7 +102,22 @@ ConstantSetWrapper::ConstantSetWrapper (ConstituentSet *in_constituents,
   myUnits = in_constants->datum.Units();
   mySimpleOffsets.levelAdd.Units (myUnits);
   maxAmplitude = origMaxAmplitude;
-  currentYear = (*in_constituents)[0].firstvalidyear();
+
+  // Harmonics file range of years may exceed that of this platform.
+  // Try valiantly to find a safe initial value.
+  {
+    unsigned b = (*in_constituents)[0].firstvalidyear().val();
+    unsigned e = (*in_constituents)[0].lastvalidyear().val();
+    if (b <= 2000 && e >= 2000)
+      currentYear = Year(2000);
+    else if (b <= 1970 && e >= 1970)
+      currentYear = Year(1970);
+    else if (b <= 2037 && e >= 2037)
+      currentYear = Year(2037);
+    else
+      currentYear = Year((b+e)/2);
+  }
+
   refresh_adjConstants_yearConstants ();
 }
 
@@ -147,10 +162,7 @@ void ConstantSetWrapper::refresh_yearConstants () {
     phases[i] = yearConstants.phases[i].rad();
   }
   epoch = Timestamp (currentYear);
-  if (currentYear.val() < globallastyear)
-    next_epoch = Timestamp (currentYear + 1);
-  else
-    next_epoch = Timestamp();  // Null timestamp
+  next_epoch = Timestamp (currentYear + 1);
 }
 
 ConstantSetWrapper::~ConstantSetWrapper () {
@@ -258,6 +270,7 @@ PredictionValue ConstantSetWrapper::datum() const {
  */
 
 // t has been changed to be seconds since the epoch.
+// (This is especially good since time_t might be a long long.)
 
 double
 ConstantSetWrapper::_time2dt_tide (long t, unsigned deriv) {
@@ -343,6 +356,7 @@ Year first_year, double blend) {
     refresh_yearConstants ();
   }
 
+  // This amount is less than a year, so a long int suffices.
   t = (in_timestamp - epoch).in_seconds();
   for (n = 0; n <= deriv; n++)
     fp[n] = _time2dt_tide(t, n);
@@ -387,13 +401,14 @@ ConstantSetWrapper::time2dt_tide (Timestamp in_timestamp, unsigned deriv)
     refresh_yearConstants ();
   }
 
+  // This amount is less than a year, so a long int suffices.
   long t = (in_timestamp - epoch).in_seconds();
 
   /*
    * If we're close to either the previous or the next
    * new years we must blend the two years' tides.
    */
-  if (t <= TIDE_BLEND_SECONDS && currentYear.val() > globalfirstyear) {
+  if (t <= TIDE_BLEND_SECONDS) {
     return blend_tide(in_timestamp, deriv, currentYear - 1,
                      ((double)t)/TIDE_BLEND_SECONDS);
   } else {
@@ -426,6 +441,7 @@ ConstantSetWrapper::time2mean (Timestamp in_timestamp)
     refresh_yearConstants ();
   }
 
+  // This amount is less than a year, so a long int suffices.
   long t = (in_timestamp - epoch).in_seconds();
 
   double dt_tide = 0.0;
@@ -437,6 +453,7 @@ ConstantSetWrapper::time2mean (Timestamp in_timestamp)
   return dt_tide;
 }
 
+#if 0
 ostream &operator<< (ostream &out, const ConstantSetWrapper &s) {
   out << "  Datum: " << s.datum() << endl;
   out << "  Constants:" << endl;
@@ -445,3 +462,4 @@ ostream &operator<< (ostream &out, const ConstantSetWrapper &s) {
       (s.origConstants)->phases[i] << endl;
   return out;
 }
+#endif
