@@ -1,4 +1,4 @@
-// $Id: Dstr.cc,v 1.1 2001/05/10 00:20:27 dave Exp $
+// $Id: Dstr.cc,v 1.3 2002/12/19 18:40:45 flaterco Exp $
 // Dstr:  Dave's String class.
 // This source is shared among several of my projects and is public domain.
 // Operations are added as needed and there is not necessarily a canonical
@@ -273,6 +273,14 @@ Dstr::operator+= (long unsigned int val) {
   return (*this);
 }
 
+Dstr&
+Dstr::operator+= (long long int val) {
+  char t[80];
+  sprintf (t, "%lld", val);
+  (*this) += t;
+  return (*this);
+}
+
 Dstr&
 Dstr::operator+= (double val) {
   char t[80];
@@ -389,7 +397,7 @@ double
 Dstr::asdouble() const {
   double t;
   if (sscanf (aschar(), "%lf", &t) != 1)
-    cerr << "Warning:  illegal conversion of Dstr to double" << endl;
+    fprintf (stderr, "Warning:  illegal conversion of Dstr to double\n");
   return t;
 }
 
@@ -419,12 +427,6 @@ Dstr::asrawchar () const {
   return theBuffer;
 }
 
-ostream &
-operator<< (ostream &out, const Dstr &val) {
-  out << val.aschar();
-  return out;
-}
-
 int operator== (const Dstr &val1, const char *val2) {
   if ((!val2) && (val1.isNull()))
     return 1;
