@@ -1,4 +1,4 @@
-// $Id: Graph.cc,v 1.1 2000/08/30 22:55:49 dave Exp $
+// $Id: Graph.cc,v 1.5 2003/02/20 15:19:56 flaterco Exp $
 /*  Graph  Abstract superclass for all graphs.
 
     Copyright (C) 1998  David Flater.
@@ -191,7 +191,7 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
 
   // now is the "now" of the graph (the time to show).
   // rightnow is the real "now" (the actual wallclock time).
-  Timestamp rightnow (time(NULL));
+  Timestamp rightnow ((time_t)(time(NULL)));
 
   // First get a list of the relevant tide events.  We need to go back
   // to synchronize internal and real timestamps.  We'll also draw one
@@ -203,7 +203,8 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
   first->tm = start;
   do
     sr->predictExactTideEvent (first->tm, Station::backward,
-      first->t_out, first->etype, first->etype_desc, first->pv);
+      first->t_out, first->etype, first->etype_desc_long,
+      first->etype_desc_short, first->pv);
   while (first->t_out >= start ||
         ((angle) && (!(isMaxMinEvent (first->etype)))));
   struct tideline *prev = first;
@@ -212,10 +213,12 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     struct tideline *next = new tideline;
     next->tm = prev->tm;
     sr->predictExactTideEvent (next->tm, Station::forward,
-      next->t_out, next->etype, next->etype_desc, next->pv);
+      next->t_out, next->etype, next->etype_desc_long,
+      next->etype_desc_short, next->pv);
     if (next->etype == prev->etype)
       sr->predictExactTideEvent (next->tm, Station::forward,
-        next->t_out, next->etype, next->etype_desc, next->pv);
+        next->t_out, next->etype, next->etype_desc_long,
+        next->etype_desc_short, next->pv);
     // Stack up for clock mode.
     if (next->t_out > now) {
       switch (next->etype) {
@@ -385,12 +388,12 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
     centerStringOnLine (myxsize/2, 0, ts);
 
     // Write next max
-    centerStringOnLine (myxsize/2, 1, nextmax->etype_desc);
+    centerStringOnLine (myxsize/2, 1, nextmax->etype_desc_long);
     nextmax->t_out.printtime (ts, sr->timeZone, settings);
     centerStringOnLine (myxsize/2, 2, ts);
 
     // Write next min
-    centerStringOnLine (myxsize/2, -3, nextmin->etype_desc);
+    centerStringOnLine (myxsize/2, -3, nextmin->etype_desc_long);
     nextmin->t_out.printtime (ts, sr->timeZone, settings);
     centerStringOnLine (myxsize/2, -2, ts);
 
@@ -398,7 +401,7 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
 
     drawTitleLine (sr->name);
 
-    // Put timestamps for tide events.
+    // Put timestamps for timestampable events.
     t = first;
     while (t) {
       Dstr ts;
@@ -411,15 +414,34 @@ void Graph::drawTides (Station *sr, Timestamp now, Angle *angle) {
 	t->t_out.printtime (ts, sr->timeZone, settings);
 	drawTimestampLabel (x, 2, ts);
 	break;
+
+      case Station::moonrise:
+      case Station::moonset:
       case Station::slackrise:
       case Station::slackfall:
       case Station::markrise:
       case Station::markfall:
+      case Station::newmoon:
+      case Station::firstquarter:
+      case Station::fullmoon:
+      case Station::lastquarter:
         drawHourTick (x, Colors::mark);
 	t->t_out.printtime (ts, sr->timeZone, settings);
 	drawTimestampLabel (x, -2, ts);
-	t->t_out.printdate (ts, sr->timeZone, settings);
-	drawTimestampLabel (x, -3, ts);
+
+        // Pre-2.6:  only draw mark/slack crossings at the bottom;
+        // include date.
+	// t->t_out.printdate (ts, sr->timeZone, settings);
+	// drawTimestampLabel (x, -3, ts);
+
+        // 2.6:  replace the date with the description and draw more
+        // sun/moon events.  Try to be smart about when to use long
+        // descriptions or short.
+        if (!is_banner() && (sr->isCurrent || sr->markLevel))
+	  drawTimestampLabel (x, -3, t->etype_desc_short);
+        else
+	  drawTimestampLabel (x, -3, t->etype_desc_long);
+
 	break;
       default:
 	;
@@ -614,3 +636,7 @@ void Graph::drawTimestampLabel (int x, int line, Dstr &ts) {
 void Graph::drawTitleLine (Dstr &title) {
   centerStringOnLine (myxsize/2, 0, title);
 }
+
+int Graph::is_banner () {
+  return 0;
+}
