@@ -1,4 +1,4 @@
-/* $Id: Settings.cc,v 1.3 2000/08/30 22:56:22 dave Exp $ */
+/* $Id: Settings.cc,v 1.2 2002/10/01 18:04:31 flaterco Exp $ */
 /*  Settings  XTide global settings
 
     Copyright (C) 1998  David Flater.
@@ -38,6 +38,8 @@ Settings::supersedeBy (const Settings &sup) {
       colors[a] = sup.colors[a];
   if (!(sup.el.isNull()))
     el = sup.el;
+  if (!(sup.fe.isNull()))
+    fe = sup.fe;
   if (!(sup.u.isNull()))
     u = sup.u;
   if (!(sup.z.isNull()))
@@ -54,6 +56,8 @@ Settings::supersedeBy (const Settings &sup) {
     nf = sup.nf;
   if (!(sup.ns.isNull()))
     ns = sup.ns;
+  if (!(sup.in.isNull()))
+    in = sup.in;
   if (!(sup.gw_isnull)) {
     gw_isnull = 0;
     gw = sup.gw;
@@ -232,12 +236,16 @@ void Settings::save() {
     fprintf (fp, "tf=\"%s\"\n", tf.aschar());
   if (el.length())
     fprintf (fp, "el=\"%s\"\n", el.aschar());
+  if (fe.length())
+    fprintf (fp, "fe=\"%s\"\n", fe.aschar());
   if (tl.length())
     fprintf (fp, "tl=\"%s\"\n", tl.aschar());
   if (nf.length())
     fprintf (fp, "nf=\"%s\"\n", nf.aschar());
   if (ns.length())
     fprintf (fp, "ns=\"%s\"\n", ns.aschar());
+  if (in.length())
+    fprintf (fp, "in=\"%s\"\n", in.aschar());
 
   if (!(gw_isnull))
     fprintf (fp, "gw=\"%u\"\n", gw);
