@@ -1,4 +1,4 @@
-// $Id: Station.cc,v 1.1 2000/08/30 22:57:04 dave Exp $
+// $Id: Station.cc,v 1.8 2003/02/20 15:19:56 flaterco Exp $
 /*  Station  Abstract superclass of ReferenceStation and SubordinateStation.
 
     Copyright (C) 1998  David Flater.
@@ -25,6 +25,7 @@ Station::Station (TideContext *in_context) {
   context = in_context;
   markLevel = NULL;
   aspect = context->settings->ga;
+  step = Interval (HOURSECONDS);
 }
 
 PredictionValue Station::minLevel() const {
@@ -35,6 +36,7 @@ PredictionValue Station::maxLevel() const {
   return constants->datum() + constants->maxAmplitude;
 }
 
+#if 0
 ostream &operator<< (ostream &out, const Station &s) {
   out << "Station name: " << s.name << endl;
   out << "Station time zone: " << s.timeZone << endl;
@@ -46,6 +48,7 @@ ostream &operator<< (ostream &out, const Station &s) {
   out << *(s.constants);
   return out;
 }
+#endif
 
 // The following block of functions is slightly revised from the code
 // delivered by Geoffrey T. Dairiki for XTide 1.  I have refrained
@@ -140,8 +143,8 @@ PredictionValue (Station::*f)(Timestamp, unsigned deriv), Direction d) {
       dt = Interval(0); // Force bisection on first step
     else if (abs(ft) > f_thresh   /* not decreasing fast enough */
 	|| (ft.val() > 0.0 ?    /* newton step would go outside bracket */
-	    (fp <= ft / (t - tl).in_seconds()) :
-	    (fp <= -ft / (tr - t).in_seconds())))
+	    (fp <= ft / (long)((t - tl).in_seconds())) :
+	    (fp <= -ft / (long)((tr - t).in_seconds()))))
       dt = Interval(0); /* Force bisection */
     else {
       /* Attempt a newton step */
@@ -383,8 +386,14 @@ Direction d)
   return Timestamp(); // Don't have a bracket, return null timestamp
 }
 
-// This has changed radically, and gotten really complicated.
+// This has changed radically and gotten really complicated.
 // (Sun and moon didn't help.)
+//
+// 2003-02-04
+// Considered replacing the whole thing with a generic sort but
+// decided just to make it more complicated again.  Extended for
+// moonrise and moonset.
+//
 // tm is "uncorrected" or "internal" timestamp.
 void Station::_predictExactTideEvent (Timestamp &tm, Direction d,
 EventType &etype_out) {
@@ -400,20 +409,31 @@ EventType &etype_out) {
     t_hilo.make_null();
     t_slack.make_null();
     t_mark.make_null();
-    t_moon.make_null();
-    t_sun.make_null();
+    t_moonrs.make_null();
+    t_moonphase.make_null();
+    t_sunrs.make_null();
   }
 
   if (!ns) {
     // Update sun
-    if (t_sun.isNull() && (!(coordinates.isNull()))) {
-      t_sun = tm;
-      find_next_sun_event (t_sun, coordinates, sun_t, d);
+    if (t_sunrs.isNull() && (!(coordinates.isNull()))) {
+      t_sunrs = tm;
+      find_next_rise_or_set (t_sunrs, coordinates, sunrs_t, d, 0);
     }
     // Update moon
-    if (t_moon.isNull()) {
-      t_moon = tm;
-      find_next_moon_event (t_moon, moon_t, d);
+    if (t_moonrs.isNull() && (!(coordinates.isNull()))) {
+      // Danger, danger.  The moonrise and moonset logic blows up if you
+      // go before 1900 or after 2099.  Leave a safety margin.
+      // This logic is straight out of skycal.cc.
+      double danger_y = 1900. + (tm.jd() - 2415019.5) / 365.25;
+      if (danger_y > 1900.1 && danger_y < 2099.9) {
+        t_moonrs = tm;
+        find_next_rise_or_set (t_moonrs, coordinates, moonrs_t, d, 1);
+      }
+    }
+    if (t_moonphase.isNull()) {
+      t_moonphase = tm;
+      find_next_moon_phase (t_moonphase, moonphase_t, d);
     }
   }
 
@@ -469,21 +489,28 @@ EventType &etype_out) {
 
   // Sun and moon -- let's try to keep this simple.  (Ha ha ha!)
   if (!ns) {
-    Timestamp *t_sunmoon;
-    EventType sunmoon_t;
-    if (t_sun.isNull()) {
-      t_sunmoon = &t_moon;
-      sunmoon_t = moon_t;
-    } else {
-      if ((d == forward && t_moon < t_sun) ||
-	  (d == backward && t_moon > t_sun)) {
-	t_sunmoon = &t_moon;
-	sunmoon_t = moon_t;
-      } else {
-	t_sunmoon = &t_sun;
-	sunmoon_t = sun_t;
+    Timestamp *t_sunmoon = &t_moonphase;
+    EventType sunmoon_t = moonphase_t;
+
+    // Make t_sunmoon point to whichever of sunrs, moonrs, or moonphase
+    // would win.  N.B., the sun and moon might never rise (particularly
+    // if we have no coordinates), but the moon always has a phase.
+    
+    if (!(t_sunrs.isNull())) {
+      if ((d == forward && t_sunrs < *t_sunmoon) ||
+	  (d == backward && t_sunrs > *t_sunmoon)) {
+	t_sunmoon = &t_sunrs;
+	sunmoon_t = sunrs_t;
       }
     }
+    if (!(t_moonrs.isNull())) {
+      if ((d == forward && t_moonrs < *t_sunmoon) ||
+	  (d == backward && t_moonrs > *t_sunmoon)) {
+	t_sunmoon = &t_moonrs;
+	sunmoon_t = moonrs_t;
+      }
+    }
+
     // While we're waiting for goto to come back into style...
     while (1) {
       if (d == forward) {
@@ -560,6 +587,8 @@ int isSunMoonEvent (Station::EventType a) {
   switch (a) {
   case Station::sunrise:
   case Station::sunset:
+  case Station::moonrise:
+  case Station::moonset:
   case Station::newmoon:
   case Station::firstquarter:
   case Station::fullmoon:
@@ -599,55 +628,79 @@ PredictionValue Station::movingMean (Timestamp tm) {
 }
 
 void Station::etypedesc (EventType etype, PredictionValue pv,
-Dstr &etype_desc) {
+Dstr &etype_desc_long, Dstr &etype_desc_short) {
   switch (etype) {
   case max:
-    etype_desc = (isCurrent ? (pv.val() >= 0.0 ? "Max Flood"
+    etype_desc_long = (isCurrent ? (pv.val() >= 0.0 ? "Max Flood"
 			       : "Min Ebb") : "High Tide");
+    etype_desc_short = (isCurrent ? (pv.val() >= 0.0 ? "MaxFl"
+			       : "MinEb") : "High");
     break;
   case min:
-    etype_desc = (isCurrent ? (pv.val() <= 0.0 ? "Max Ebb"
+    etype_desc_long = (isCurrent ? (pv.val() <= 0.0 ? "Max Ebb"
                                : "Min Flood") : "Low Tide");
+    etype_desc_short = (isCurrent ? (pv.val() <= 0.0 ? "MaxEb"
+                               : "MinFl") : "Low");
     break;
   case slackrise:
-    etype_desc = "Slack, Flood Begins";
+    etype_desc_long = "Slack, Flood Begins";
+    etype_desc_short = "Slack";
     break;
   case slackfall:
-    etype_desc = "Slack, Ebb Begins";
+    etype_desc_long = "Slack, Ebb Begins";
+    etype_desc_short = "Slack";
     break;
   case markrise:
-    etype_desc = (isCurrent ? (pv.val() <= 0.0 ? "Mark, Ebb Decreasing"
+    etype_desc_long = (isCurrent ? (pv.val() <= 0.0 ? "Mark, Ebb Decreasing"
 			       : "Mark, Flood Increasing")
 		  : "Mark Rising");
+    etype_desc_short = "Mark";
     break;
   case markfall:
-    etype_desc = (isCurrent ? (pv.val() >= 0.0 ? "Mark, Flood Decreasing"
+    etype_desc_long = (isCurrent ? (pv.val() >= 0.0 ? "Mark, Flood Decreasing"
 			       : "Mark, Ebb Increasing")
 		  : "Mark Falling");
+    etype_desc_short = "Mark";
     break;
   case sunrise:
-    etype_desc = "Sunrise";
+    etype_desc_long = "Sunrise";
+    etype_desc_short = "Srise";
     break;
   case sunset:
-    etype_desc = "Sunset";
+    etype_desc_long = "Sunset";
+    etype_desc_short = "Sset";
+    break;
+  case moonrise:
+    etype_desc_long = "Moonrise";
+    etype_desc_short = "Mrise";
+    break;
+  case moonset:
+    etype_desc_long = "Moonset";
+    etype_desc_short = "Mset";
     break;
   case newmoon:
-    etype_desc = "New Moon";
+    etype_desc_long = "New Moon";
+    etype_desc_short = "NewM";
     break;
   case firstquarter:
-    etype_desc = "First Quarter";
+    etype_desc_long = "First Quarter";
+    etype_desc_short = "1stQ";
     break;
   case fullmoon:
-    etype_desc = "Full Moon";
+    etype_desc_long = "Full Moon";
+    etype_desc_short = "FullM";
     break;
   case lastquarter:
-    etype_desc = "Last Quarter";
+    etype_desc_long = "Last Quarter";
+    etype_desc_short = "LastQ";
     break;
   default:
     assert (0);
   }
 }
 
+// N.B., see the .cc of xxDrawable::global_redraw() regarding an
+// important dependency on how this is implemented.
 Station *Station::clone() {
   Station *s = stationRef->load(context);
   if (markLevel)
@@ -655,6 +708,7 @@ Station *Station::clone() {
   else
     s->markLevel = NULL;
   s->aspect = aspect;
+  s->step = step;
   if (s->myUnits != myUnits)
     s->setUnits (myUnits);
   return s;
