@@ -1,4 +1,4 @@
-// $Id: StationIndex.cc,v 1.2 2001/03/23 22:01:33 dave Exp $
+// $Id: StationIndex.cc,v 1.2 2003/01/17 19:14:15 flaterco Exp $
 /*  StationIndex  Collection of StationRefs.
 
     Copyright (C) 1998  David Flater.
@@ -56,8 +56,13 @@ unsigned long maxlength) {
   char **temp = (char **) malloc ((maxlength+1) * sizeof (char *));
   unsigned long i;
   for (i=startat; i<startat+maxlength; i++) {
-    Dstr fname;
-    stationrefs[i]->shortHarmonicsFileName (fname);
+    Dstr styp;
+
+    if (stationrefs[i]->is_reference_station())
+      styp = "Ref";
+    else
+      styp = "Sub";
+
     char cbuf[20];
     char tempbuf[121];
     if (stationrefs[i]->coordinates.isNull())
@@ -66,9 +71,9 @@ unsigned long maxlength) {
       sprintf (cbuf, "%8.4f %9.4f",
                              stationrefs[i]->coordinates.lat(),
                              stationrefs[i]->coordinates.lng());
-    sprintf (tempbuf, "%-80.80s %-20.20s %18.18s",
+    sprintf (tempbuf, "%-80.80s %-4.4s %18.18s",
                              stationrefs[i]->name.aschar(),
-                             fname.aschar(),
+                             styp.aschar(),
                              cbuf);
     temp[i-startat] = strdup (tempbuf);
   }
