@@ -1,4 +1,4 @@
-// $Id: SubordinateStation.cc,v 1.1 2001/03/23 21:54:56 dave Exp $
+// $Id: SubordinateStation.cc,v 1.2 2003/02/04 15:53:46 flaterco Exp $
 /*  SubordinateStation  Implementation for offset station.
 
     Copyright (C) 1998  David Flater.
@@ -178,8 +178,8 @@ SubordinateStation::~SubordinateStation() {
 // For reference station or simple offsets, they are identical.
 // pv_out is undefined if the event is a sun or moon event.
 void SubordinateStation::predictExactTideEvent (Timestamp &tm, Direction d,
-Timestamp &t_out, EventType &etype_out, Dstr &etype_desc,
-PredictionValue &pv_out) {
+Timestamp &t_out, EventType &etype_out, Dstr &etype_desc_long,
+Dstr &etype_desc_short, PredictionValue &pv_out) {
   _predictExactTideEvent (tm, d, etype_out);
 
   t_out = tm;
@@ -232,7 +232,7 @@ PredictionValue &pv_out) {
   // The etype_desc figuring needs to be delayed until after everything
   // so that "Max Flood" versus "Min Ebb" will be correct when offsets
   // bump something over the line.
-  etypedesc (etype_out, pv_out, etype_desc);
+  etypedesc (etype_out, pv_out, etype_desc_long, etype_desc_short);
 }
 
 
