@@ -1,4 +1,4 @@
-// 	$Id: TideContext.cc,v 1.8 2001/03/23 21:52:40 dave Exp $	
+// $Id: TideContext.cc,v 1.6 2003/02/04 15:53:46 flaterco Exp $
 /*  TideContext  In lieu of global variables and functions.
 
     Copyright (C) 1998  David Flater.
@@ -53,7 +53,7 @@ Settings *in_settings) {
     }
   }
   if (hfile_path.isNull())
-    hfile_path = "harmonics";
+    hfile_path = "harmonics.tcd";
   harmonicsPath = new HarmonicsPath (hfile_path);
 
   homedir = getenv ("HOME");
@@ -131,66 +131,14 @@ StationIndex *TideContext::stationIndex () {
   return myStationIndex;
 }
 
-StationRef *TideContext::doFastloadHarmonicsFile (Dstr &fname,
-const Dstr &stationname) {
-  HarmonicsFile h (fname, settings);
-  StationRef *s = h.getStationRef (stationname);
-  if (s)
-    s->fastload = 1;
-  return s;
-}
-
-// Yet another duplication of this horrible block of code....
-StationRef *TideContext::fastload (const Dstr &name) {
-  if (myStationIndex) {
-    return (*myStationIndex)[name];
-  } else {
-    for (unsigned i=0; i<harmonicsPath->length(); i++) {
-      struct stat s;
-      if (stat ((*harmonicsPath)[i].aschar(), &s) == 0) {
-        if (S_ISDIR (s.st_mode)) {
-          Dstr dname ((*harmonicsPath)[i]);
-          dname += '/';
-          DIR *dirp = opendir (dname.aschar());
-          if (!dirp)
-            xperror (dname.aschar());
-          else {
-            struct dirent *dp;
-	    for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp)) {
-              Dstr fname (dp->d_name);
-	      if (fname == "." || fname == "..")
-		continue;
-	      else {
-                fname *= dname;
-                Dstr *qfname = new Dstr(fname);
-                StationRef *sref = doFastloadHarmonicsFile (*qfname, name);
-                if (sref)
-                  return sref; // Can't delete qfname
-                else
-                  delete qfname;
-              }
-	    }
-	    closedir(dirp);
-          }
-        } else {
-          StationRef *sref = doFastloadHarmonicsFile ((*harmonicsPath)[i], name);
-          if (sref)
-            return sref;
-        }
-      } else
-        xperror ((*harmonicsPath)[i].aschar());
-    }
-    return NULL;
-  }
-}
-
 void
 TideContext::predictText (Station *station, Station::Direction dir,
 Timestamp &tm, Dstr &line, Station::EventType &etype, Timestamp *t_out) {
   Timestamp ts;
   PredictionValue pv;
-  Dstr etype_desc, time_print, pv_print;
-  station->predictExactTideEvent (tm, dir, ts, etype, etype_desc, pv);
+  Dstr etype_desc_long, etype_desc_short, time_print, pv_print;
+  station->predictExactTideEvent (tm, dir, ts, etype, etype_desc_long,
+    etype_desc_short, pv);
   if (t_out)
     (*t_out) = ts;
   ts.print (time_print, station->timeZone, settings);
@@ -200,7 +148,7 @@ Timestamp &tm, Dstr &line, Station::EventType &etype, Timestamp *t_out) {
   line += " ";
   line += pv_print;
   line += "  ";
-  line += etype_desc;
+  line += etype_desc_long;
 
   // This is replaced by code in text_boilerplate.
 #if 0
@@ -391,7 +339,7 @@ Timestamp end_tm, Dstr &text_out) {
   if (station->isCurrent)
     text_out += "Permanent current: ";
   else
-    text_out += "Mean Sea Level: ";
+    text_out += "Mean Tide Level: ";
   meanl.print (temp);
   text_out += temp;
   text_out += "\n\n";
@@ -401,10 +349,10 @@ Timestamp end_tm, Dstr &text_out) {
   while (tm <= end_tm) {
     PredictionValue pv_out;
     Station::EventType etype;
-    Dstr etype_desc;
+    Dstr etype_desc_long, etype_desc_short;
     Timestamp t_out;
     station->predictExactTideEvent (tm, Station::forward, t_out,
-      etype, etype_desc, pv_out);
+      etype, etype_desc_long, etype_desc_short, pv_out);
     if (tm > end_tm)
       break;
     if (!isSunMoonEvent (etype)) {
@@ -449,13 +397,18 @@ Timestamp end_tm, Dstr &text_out) {
 }
 
 void
-TideContext::rawMode (Station *station, Timestamp start_tm, Timestamp end_tm,
-		      Interval step, Dstr &text_out) {
+TideContext::rareModes (Station *station, Timestamp start_tm, Timestamp end_tm,
+		      Interval step, Dstr &text_out, int cook_time) {
   text_out = (char *)NULL;
   assert (step > Interval(0));
   assert (start_tm <= end_tm);
   for (Timestamp t = start_tm; t <= end_tm; t += step) {
-    text_out += t.timet();
+    if (cook_time) {
+      Dstr time_print;
+      t.print (time_print, station->timeZone, settings);
+      text_out += time_print;
+    } else
+      text_out += t.timet();
     text_out += " ";
     text_out += station->predictApproximate(t).val();
     text_out += "\n";
@@ -464,24 +417,27 @@ TideContext::rawMode (Station *station, Timestamp start_tm, Timestamp end_tm,
 
 void TideContext::listModePlain (Dstr &text_out) {
   text_out = "Location list generated ";
-  Timestamp now (time(NULL));
+  Timestamp now ((time_t)(time(NULL)));
   Dstr tempnow, tz;
   tz = "UTC0";
   now.print (tempnow, tz, settings);
   text_out += tempnow;
   text_out += "\n\n";
-  int namewidth = (int)(settings->tw) - 45;
+  int namewidth = (int)(settings->tw) - 28;
   if (namewidth < 10)
     namewidth = 10;
   StationIndex *si = stationIndex();
   char fmt[80];
-  sprintf (fmt, "%%-%d.%ds %%-20.20s %%-23.23s\n", namewidth, namewidth);
+  sprintf (fmt, "%%-%d.%ds %%-3.3s %%-23.23s\n", namewidth, namewidth);
   char *buf = (char *) malloc (settings->tw + 30);
   for (unsigned i=0; i<si->length(); i++) {
-    Dstr fname, c;
-    (*si)[i]->shortHarmonicsFileName (fname);
+    Dstr styp, c;
+    if ((*si)[i]->is_reference_station())
+      styp = "Ref";
+    else
+      styp = "Sub";
     (*si)[i]->coordinates.print (c, 1);
-    sprintf (buf, fmt, (*si)[i]->name.aschar(), fname.aschar(), c.aschar());
+    sprintf (buf, fmt, (*si)[i]->name.aschar(), styp.aschar(), c.aschar());
     text_out += buf;
   }
   free (buf);
@@ -489,7 +445,7 @@ void TideContext::listModePlain (Dstr &text_out) {
 
 void
 TideContext::startLocListHTML (Dstr &d) {
-  d += "<p><table>\n<tr><th>Location</th><th>File</th>\n\
+  d += "<p><table>\n<tr><th>Location</th><th>Type</th>\n\
 <th>Coordinates</th></tr>";
 }
 
@@ -504,11 +460,10 @@ TideContext::listLocationHTML (Dstr &d, StationRef *sr) {
   d += "<tr><td>";
   d += sr->name;
   d += "</td><td>";
-  {
-    Dstr fname;
-    sr->shortHarmonicsFileName (fname);
-    d += fname;
-  }
+  if (sr->is_reference_station())
+    d += "Ref";
+  else
+    d += "Sub";
   d += "</td><td>";
   Dstr tempc;
   sr->coordinates.print (tempc);
@@ -518,7 +473,7 @@ TideContext::listLocationHTML (Dstr &d, StationRef *sr) {
 
 void TideContext::listModeHTML (Dstr &text_out) {
   text_out = "<p> Location list generated ";
-  Timestamp now (time(NULL));
+  Timestamp now ((time_t)(time(NULL)));
   Dstr tempnow, tz;
   tz = "UTC0";
   now.print (tempnow, tz, settings);
