@@ -1,4 +1,4 @@
-// 	$Id: Timestamp.cc,v 1.8 2001/03/23 01:28:12 dave Exp $	
+// $Id: Timestamp.cc,v 1.7 2003/02/20 15:19:56 flaterco Exp $
 // Timestamp:  encapsulation for all manner of time-related services needed
 // by XTide.
 
@@ -27,49 +27,172 @@
 //    time_t 0x00000000 = 1970-01-01 00:00:00Z
 //    time_t 0x7FFFFFFF = 2038-01-19 03:14:07Z
 //    time_t 0x80000000 = 1901-12-13 20:45:52Z
+// However, Posix does not require it to be signed, so the safe range
+// is only 1970 to 2037.
 
-// These static vars are used to speed up various operations.
-static int years_init = 0;
-static time_t years[globallastyear - globalfirstyear + 2];
-static time_t startoftime, endoftime;
 
-// This should be called in every constructor.
-void
-Timestamp::init_years () {
-  if (!years_init) {
-    years_init = 1;
-    // WARNING note usage of globallastyear+1
-    for (unsigned i=globalfirstyear; i<=globallastyear+1; i++) {
-      struct tm ht;
-      ht.tm_year = i - 1900;
-      ht.tm_sec = ht.tm_min = ht.tm_hour = ht.tm_mon = 0;
-      ht.tm_mday = 1;
-      years[i-globalfirstyear] = tm2utc (ht);
+#ifdef TIME_WORKAROUND
+
+// ************************************
+//     Begin derivative of GNU code
+// ************************************
+
+// When TIME_WORKAROUND is engaged, time_t is redefined as a signed
+// 64-bit integer in common.hh.  It is used below anytime a signed
+// 64-bit integer is required, whether or not it's actually a
+// timestamp.
+
+// This function was lifted from the file time/offtime.c in
+// glibc-2.3.1, made standalone, munged in a futile effort to avoid
+// overflows, and renamed to avoid conflicting with any such function
+// that is present on the compiling system.
+
+// Modifications are flagged with DWF.
+
+/* Copyright (C) 1991, 1993, 1997, 1998, 2002 Free Software Foundation, Inc.
+   This file is part of the GNU C Library.
+
+   The GNU C Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2.1 of the License, or (at your option) any later version.
+
+   The GNU C Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with the GNU C Library; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+   02111-1307 USA.  */
+
+// DWF: commented out
+// #include <errno.h>
+// #include <time.h>
+
+// DWF: incorporated from glibc-2.3.1 time/mktime.c
+/* Nonzero if YEAR is a leap year (every 4 years,
+   except every 100th isn't, and every 400th is).  */
+#define __isleap(year) \
+  ((year) % 4 == 0 && ((year) % 100 != 0 || (year) % 400 == 0))
+
+// DWF: incorporated from glibc-2.3.1 time/mktime.c
+/* How many days come before each month (0-12).  */
+const unsigned short int __mon_yday[2][13] =
+  {
+    /* Normal years.  */
+    { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365 },
+    /* Leap years.  */
+    { 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366 }
+  };
+
+#define	SECS_PER_HOUR	(60 * 60)
+#define	SECS_PER_DAY	(SECS_PER_HOUR * 24)
+
+/* Compute the `struct tm' representation of *T,
+   offset OFFSET seconds east of UTC,
+   and store year, yday, mon, mday, wday, hour, min, sec into *TP.
+   Return nonzero if successful.  */
+// DWF: renamed, converted to C++ declaration syntax
+int xtide_offtime (const time_t *t, long int offset, struct tm *tp)
+{
+  time_t days, y; // DWF: 32 bits doesn't cut it
+  long int rem;
+  const unsigned short int *ip;
+
+  days = *t / SECS_PER_DAY;  // DWF: here's why
+  rem = *t % SECS_PER_DAY;
+  rem += offset;
+  while (rem < 0)
+    {
+      rem += SECS_PER_DAY;
+      --days;
     }
-    startoftime = years[0];
-    endoftime = years[globallastyear+1-globalfirstyear] - 1;
-  }
+  while (rem >= SECS_PER_DAY)
+    {
+      rem -= SECS_PER_DAY;
+      ++days;
+    }
+  tp->tm_hour = rem / SECS_PER_HOUR;
+  rem %= SECS_PER_HOUR;
+  tp->tm_min = rem / 60;
+  tp->tm_sec = rem % 60;
+  /* January 1, 1970 was a Thursday.  */
+  tp->tm_wday = (4 + days) % 7;
+  if (tp->tm_wday < 0)
+    tp->tm_wday += 7;
+  y = 1970;
+
+#define DIV(a, b) ((a) / (b) - ((a) % (b) < 0))
+#define LEAPS_THRU_END_OF(y) (DIV (y, 4) - DIV (y, 100) + DIV (y, 400))
+
+  while (days < 0 || days >= (__isleap (y) ? 366 : 365))
+    {
+      /* Guess a corrected year, assuming 365 days per year.  */
+      // DWF: 64 bits
+      time_t yg = y + days / 365 - (days % 365 < 0);
+
+      /* Adjust DAYS and Y to match the guessed year.  */
+      days -= ((yg - y) * 365
+	       + LEAPS_THRU_END_OF (yg - 1)
+	       - LEAPS_THRU_END_OF (y - 1));
+      y = yg;
+    }
+  tp->tm_year = y - 1900;
+  if (tp->tm_year != y - 1900)
+    {
+      /* The year cannot be represented due to overflow.  */
+      // DWF: Don't know where __set_errno came from but I don't have it
+      // __set_errno (EOVERFLOW);
+      errno = EOVERFLOW;
+      return 0;
+    }
+  tp->tm_yday = days;
+  ip = __mon_yday[__isleap(y)];
+  for (y = 11; days < (long int) ip[y]; --y)
+    continue;
+  days -= ip[y];
+  tp->tm_mon = y;
+  tp->tm_mday = days + 1;
+  return 1;
 }
 
+// **********************************
+//     End derivative of GNU code
+// **********************************
+
+// Local version of gmtime, complete with internal static buffer.
+// Returns null in case of trouble.
+struct tm *xtide_gmtime (const time_t *t) {
+  static struct tm sstm;
+  if (!(xtide_offtime (t, 0, &sstm)))
+    return NULL;
+  return &sstm;
+}
+
+// Redirection
+#define gmtime xtide_gmtime
+#define localtime xtide_gmtime
+
+// Mktime is only used once and is redirected at the invocation.
+
+#endif
+
 Timestamp::Timestamp () {
-  init_years();
   is_null = 1;
 }
 
 // Create a Timestamp from a Posix timestamp.
 Timestamp::Timestamp (time_t in_posixtime) {
-  init_years();
   is_null = 0;
   posixtime = in_posixtime;
-  check_epoch (posixtime);
 }
 
 // Create a Timestamp for the specified Julian date.
 Timestamp::Timestamp (double jd) {
-  init_years();
   is_null = 0;
   posixtime = (time_t)((jd - beginning_of_time) * DAYSECONDS);
-  check_epoch (posixtime);
 }
 
 // Convert to Julian date.
@@ -80,33 +203,23 @@ double Timestamp::jd() {
 // Create a Timestamp for the beginning of the specified year in UTC
 // (YEAR-01-01 00:00:00Z)
 Timestamp::Timestamp (Year in_year) {
-  init_years();
   is_null = 0;
   posixtime = year2utc (in_year);
 }
 
 Timestamp::Timestamp (struct tm in_utctime) {
-  init_years();
   is_null = 0;
   posixtime = tm2utc (in_utctime);
-  check_epoch (posixtime);
 }
 
 Timestamp::Timestamp (Year in_year, double fraction) {
-  init_years();
   is_null = 0;
 
   assert (fraction >= 0.0 && fraction <= 1.0);
 
-  // To make this work for the year 2037, I need to subvert the epoch
-  // checking.  (Boo, hiss.)
-
   time_t y1 = year2utc (in_year);
-  // WARNING note usage of globallastyear+1
-  time_t y2 = year2utc (in_year.val() + 1);
-  posixtime = y1 + (time_t) (fraction * (y2 - y1)); // Dubious casting
-
-  check_epoch (posixtime);  // No sneaking 2037 + 1.0
+  time_t y2 = year2utc (in_year + 1);
+  posixtime = y1 + (time_t) (fraction * (y2 - y1));
 }
 
 int
@@ -114,52 +227,25 @@ Timestamp::isNull() {
   return is_null;
 }
 
-/* Verify that t falls within the legal epoch. */
-void
-Timestamp::check_epoch (time_t t)
-{
-  if (t < startoftime || t > endoftime) {
-    // This is actually the old implementation -- it gets Year to
-    // generate the error.
-    // SGI compiler barfs:  "dependent statement may not be a declaration"
-    // Year junk (((gmtime (&t))->tm_year) + 1900);
-    unsigned gy = ((gmtime (&t))->tm_year) + 1900;
-    Year junk (gy);
-  }
-}
-
 // A simple experiment revealed that the call to gmtime to determine
-// the year of a timestamp was adding 50% to the time that the
-// innermost tide-predicting function took to complete... so I've sped
-// this up.  This was what got me started on all those static vars.
-//   return Year (((gmtime (&posixtime))->tm_year) + 1900);
+// the year of a timestamp adds 50% to the time that the innermost
+// tide-predicting function takes to complete.  At some point I
+// optimized this to use a lookup table for the years 1970 to 2037,
+// but that became a liability when we started going outside of that
+// epoch.  So we are back to the slow but simple way.
 
 Year Timestamp::year() {
   assert (!is_null);
-  unsigned i;
-  for (i=globallastyear; i>=globalfirstyear; i--)
-    if (posixtime >= year2utc (i)) {
-#ifdef SUPER_ULTRA_VERBOSE_DEBUGGING
-      cerr << "UTC year() for timestamp " <<
-        ctime (&posixtime) << "   returning " << i << endl;
-      assert (i == (((gmtime (&posixtime))->tm_year) + 1900));
-#endif
-      return Year(i);
-    }
-  assert (0);
-  // Silence bogus SGI compiler warning
-  return Year(i);
+  return Year (((gmtime (&posixtime))->tm_year) + 1900);
 }
 
-// This works on globallastyear + 1 for the benefit of the weird constructor.
-time_t
-Timestamp::year2utc (unsigned in_year) {
-  assert (in_year >= globalfirstyear && in_year <= globallastyear+1);
-  return years[in_year-globalfirstyear];
-}
-time_t
-Timestamp::year2utc (Year in_year) {
-  return year2utc (in_year.val());
+// Likewise, year2utc was using the lookup table.
+time_t Timestamp::year2utc (Year in_year) {
+  struct tm ht;
+  ht.tm_year = in_year.val() - 1900;
+  ht.tm_sec = ht.tm_min = ht.tm_hour = ht.tm_mon = 0;
+  ht.tm_mday = 1;
+  return tm2utc (ht);
 }
 
 // Compare two struct tm's according to the fields tm_year, tm_mon,
@@ -194,26 +280,41 @@ Timestamp::tm2utc (struct tm ht)
   int loopcounter = sizeof(time_t) * 8;
   time_t thebit = ((time_t)1) << (loopcounter-1);
 
-  // Check year first (gives a more informative error message).
-  // No, don't.
-  // See comments in Timestamp::Timestamp (Year in_year, double fraction)
-  // Year junk (ht.tm_year + 1900);
-
-  // If time_t is signed, skip the sign bit.
+  // Are we signed?
   if (thebit < (time_t)0) {
-    // You can't just shift thebit right because it propagates the sign bit.
-    loopcounter--;
+    switch (loopcounter) {
+    case 32:
+      // Get this sign thing out of the way.
+      if (ht.tm_year + 1900 < 1970)
+	guess |= 0x80000000;
+      loopcounter = 31;
+      break;
+    case 64:
+      // Had a lot of trouble with overflows.  40 bits should suffice for
+      // all reasonable requests.
+      if (ht.tm_year + 1900 < 1970)
+	guess |= 0xFFFFFF0000000000LL;
+      loopcounter = 40;
+      break;
+    default:
+      // Wha?!?
+      // Skip the sign bit and hope for the best.
+      // You can't just shift thebit right because it propagates the sign bit.
+      loopcounter--;
+    }
     thebit = ((time_t)1) << (loopcounter-1);
     assert (thebit > (time_t)0);
+  } else {
+    // We are unsigned.
+    // Assuming no overflows, no action required here.
+    ;
   }
 
   int compare;
   struct tm* gm;
   for (; loopcounter; loopcounter--) {
     time_t newguess = guess | thebit;
-    // Falk Hueffner reports that on his Alpha (BSD?) time_t is 64 bits
-    // and gmtime returns NULL if the time_t is out of a certain range.
-    // So if gmtime fails, we assume that we are too far in the future.
+    // If gmtime fails, we assume that we are too far in the future.
     gm = gmtime(&newguess);
     if (!(compare = (gm ? compare_tm (*gm, ht) : 1)))
       return newguess;
@@ -239,6 +340,7 @@ Timestamp::tm2utc (struct tm ht)
   return guess;
 }
 
+#if 0
 // Output timestamp in ISO 8601 format (UTC).
 ostream &operator<< (ostream &out, Timestamp &in_time) {
   assert (!in_time.is_null);
@@ -248,17 +350,14 @@ ostream &operator<< (ostream &out, Timestamp &in_time) {
   out << tempbuf;
   return out;
 }
+#endif
 
 // Subtract b from a.
 Interval operator- (Timestamp a, Timestamp b) {
   assert (!a.is_null);
   assert (!b.is_null);
-
-  // This ain't gonna work if time_t is 64 bits and long is only 32.
-  // (Functionality with 64-bit time_t is unknown.)
-  if (sizeof (time_t) > sizeof (long))
-    barf (TIME_T_NOT_AN_INT);
-
+  if (sizeof(interval_rep_t) > sizeof(time_t))
+    return Interval ((interval_rep_t)(a.posixtime) - (interval_rep_t)(b.posixtime));
   return Interval (a.posixtime - b.posixtime);
 }
 
@@ -293,18 +392,12 @@ int operator<= (Timestamp a, Timestamp b) {
 Timestamp &Timestamp::operator+= (Interval b) {
   assert (!(is_null));
   posixtime += b.in_seconds();
-  check_epoch (posixtime);
   return (*this);
 }
 
 Timestamp &Timestamp::operator-= (Interval b) {
   assert (!(is_null));
-  if (posixtime < b.in_seconds()) {
-    Dstr details ("Underflow on subtraction of interval from timestamp");
-    barf (YEAR_OUT_OF_RANGE, details);
-  }
   posixtime -= b.in_seconds();
-  check_epoch (posixtime);
   return (*this);
 }
 
@@ -405,7 +498,6 @@ void Timestamp::printdayheading (Dstr &out_heading, const Dstr &timezone,
 
 Timestamp::Timestamp (const Dstr &in_iso, const Dstr &timezone,
 Settings *settings) {
-  init_years();
   is_null = 1;
   struct tm temp_tm;
   temp_tm.tm_sec = 0;
@@ -423,14 +515,41 @@ Settings *settings) {
   temp_tm.tm_mon -= 1;
   temp_tm.tm_isdst = -1;
   install_time_zone (timezone, settings);
+  errno = 0;
+#ifdef TIME_WORKAROUND
+  posixtime = tm2utc (temp_tm);
+#else
   posixtime = mktime (&temp_tm);
-  if (posixtime != (time_t)(-1))
+#endif
+  if (errno == 0)
     is_null = 0;
 }
 
 
 /**************************************************************************/
 
+#ifdef TIME_WORKAROUND
+
+void Timestamp::install_time_zone (const Dstr &timeZone_in, Settings *settings)
+{
+  // The only use of time zones is by strftime for %Z
+  static char env_string[80];
+  static int installed = 0;
+  if (!installed) {
+    installed = 1;
+    strcpy (env_string, "TZ=UTC0");
+    assert (putenv (env_string) == 0);
+    tzset ();
+  }
+}
+
+int Timestamp::zoneinfo_doesnt_suck(Settings *settings) {
+  // This function is unnecessary when TIME_WORKAROUND is engaged.
+  return 1;
+}
+
+#else
+
 /* Declarations for zoneinfo compatibility hack */
 
 /* It's not worth detecting or supporting POSIX because all the platforms
@@ -558,7 +677,7 @@ static char *subs[][4] = {
 {":America/Halifax",            ":Canada/Atlantic", "AST4ADT", "AST4ADT"},
 {":America/Havana",             ":Cuba", "CST5", "CST5"},
 {":America/Hermosillo",         "MST7", "MST7", "MST7"},
-{":America/Iqaluit",            ":Canada/Central", "CST6CDT", "CST6CDT"},
+{":America/Iqaluit",            ":Canada/Eastern", "EST5EDT", "EST5EDT"},
 {":America/Jamaica",            ":Jamaica", "EST5EDT", "EST5EDT"},
 {":America/La_Paz",             "AST4", "AST4", "AST4"},
 {":America/Lima",               "EST5", "EST5", "EST5"},
@@ -568,7 +687,7 @@ static char *subs[][4] = {
 {":America/Manaus",             ":Brazil/West", "WST4", "WST4"},
 {":America/Martinique",         "AST4", "AST4", "AST4"},
 {":America/Mazatlan",           ":Mexico/BajaSur", "MST7MDT", "MST7MDT"},
-{":America/Mexico_City",        ":Mexico/General", "CST6CDT", "CST6CDT"},
+{":America/Mexico_City",        "CST6", "CST6", "CST6"},
 {":America/Miquelon",           "SPST3SPDT", "SPST3SPDT", "SST3SDT"},
 {":America/Montevideo",         "EST3", "EST3", "EST3"},
 {":America/Montreal",           ":Canada/Eastern", "EST5EDT", "EST5EDT"},
@@ -860,7 +979,7 @@ static char *subs[][4] = {
 
 {":Mexico/BajaNorte",           ":Mexico/BajaNorte", "PST8PDT", "PST8PDT"},
 {":Mexico/BajaSur",             ":Mexico/BajaSur", "MST7MDT", "MST7MDT"},
-{":Mexico/General",             ":Mexico/General", "CST6CDT", "CST6CDT"},
+{":Mexico/General",             "CST6", "CST6", "CST6"},
 
 {":Brazil/West",                ":Brazil/West", "WST4", "WST4"},
 {":Brazil/DeNoronha",           ":Brazil/DeNoronha", "FST2", "FST2"},
@@ -1002,6 +1121,9 @@ Timestamp::zoneinfo_doesnt_suck(Settings *settings) {
   return (tzlevel == NEWZONEINFO);
 }
 
+#endif
+
+
 /* Find the previous hour-transition from a specified time.  If a DST
    change skips over the transition, then we skip it too. */
 void
