@@ -1,4 +1,4 @@
-// 	$Id: errors.cc,v 1.6 2001/03/23 01:35:40 dave Exp $	
+// 	$Id: errors.cc,v 1.4 2002/12/19 18:40:45 flaterco Exp $	
 // errors:  Global functions for bailing out
 
 /*
@@ -48,7 +48,7 @@ void log (const char *message, int priority) {
     if (daemon_mode)
       syslog (priority, "%s", message);
     else
-      cerr << message << endl;
+      fprintf (stderr, "%s\n", message);
   }
 }
 
@@ -67,25 +67,15 @@ int fatal = 1) {
   switch (err) {
   case YEAR_OUT_OF_RANGE:
     errout += "YEAR_OUT_OF_RANGE\n\
-XTide depends on Unix system services that only work reliably between\n\
-the years ";
-    errout += globalfirstyear;
-    errout += " and ";
-    errout += globallastyear;
-    errout += ".  \
-Some operation has been initiated that needs\n\
-to access a year outside of this range in order to complete.";
+Some operation has attempted to access time before 1 A.D. or after\n\
+4000 A.D.  There's no point even trying to predict tides over that\n\
+span of time.";
     break;
   case NULL_YEAR:
     errout += "NULL_YEAR\n\
 An attempt was made to access the value of an instance of class Year\n\
 before its value was set.  This should never happen.";
     break;
-  case TIME_T_NOT_AN_INT:
-    errout += "TIME_T_NOT_AN_INT\n\
-XTide cannot run on this platform because the time_t (Unix timestamp)\n\
-data type is bigger than the long integer data type.";
-    break;
   case TM2UTC_FAILED:
     errout += "TM2UTC_FAILED\n\
 XTide was unable to convert an informal time specification into a\n\
@@ -93,7 +83,8 @@ legal Unix timestamp.  This probably means that there is something\n\
 wrong with the informal time specification, such as referring to a\n\
 point in time that does not exist (like February 30th).  It may also\n\
 mean that the requested time is outside of the Unix epoch, which only\n\
-reliably includes the years from 1970 through 2037.";
+reliably includes the years from 1970 through 2037.  For the workaround,\n\
+see http://www.flaterco.com/xtide/time_t.html.";
     break;
   case MKTIME_FAILED:
     errout += "MKTIME_FAILED\n\
@@ -102,7 +93,8 @@ legal Unix timestamp.  This probably means that there is something\n\
 wrong with the informal time specification, such as referring to a\n\
 point in time that does not exist (like February 30th).  It may also\n\
 mean that the requested time is outside of the Unix epoch, which only\n\
-reliably includes the years from 1970 through 2037.";
+reliably includes the years from 1970 through 2037.  For the workaround,\n\
+see http://www.flaterco.com/xtide/time_t.html.";
     break;
   case INVALID_CONSTITUENT:
     errout += "INVALID_CONSTITUENT\n\
@@ -130,8 +122,7 @@ If the harmonics file has been updated, just restart xtide and it will be fine."
     errout += "NO_HFILE_PATH\n\
 If /etc/xtide.conf is not provided, you must set the environment variable\n\
 HFILE_PATH to point to your harmonics files.  Example:\n\
-  export HFILE_PATH=/usr/local/share/xtide/harmonics.txt:\\\n\
-  /usr/local/share/xtide/offsets.xml\n\
+  export HFILE_PATH=/usr/local/share/xtide/harmonics.tcd\n\
 Please refer to the documentation for usage of /etc/xtide.conf.";
     break;
   case IMPOSSIBLE_CONVERSION:
@@ -247,6 +238,14 @@ The -b and -e command line switches expect timestamps to be in the format\n\
 A mark level was specified for a subordinate station whose offsets are too\n\
 complex to permit accurate mark level computation.";
     break;
+  case BAD_OR_AMBIGUOUS_COMMAND_LINE:
+    errout += "BAD_OR_AMBIGUOUS_COMMAND_LINE\n\
+The command line could not be rationalized.  Probably you have provided an\n\
+unrecognized switch or invalid argument.  However, you can also get this\n\
+error by using ambiguous syntax.  For example, the shorthand -lw5 could mean\n\
+\"set the line width to 5\" (-lw 5) or it could mean \"load the location\n\
+named w5\" (-l w5).  Sorry I can't be more specific.";
+    break;
   default:
     assert (0);
   }
@@ -275,7 +274,7 @@ barf (enum tideerr err, Dstr &details, int fatal)
   if (daemon_mode)
     log (errout, LOG_ERR);
   else
-    cerr << errout;
+    fprintf (stderr, "%s", errout.aschar());
 
 #ifdef XTIDE_X11
   if (mycontext) {
