@@ -1,4 +1,4 @@
-// $Id: tide.cc,v 1.1 2000/08/30 22:59:16 dave Exp $
+// $Id: tide.cc,v 1.5 2002/12/19 18:40:45 flaterco Exp $
 /*  tide  Command-line client for dusty old TTYs and line printers.
 
     Copyright (C) 1998  David Flater.
@@ -34,10 +34,7 @@ FILE *outfp, char mode, char format) {
   StationRef *sr;
   CommandLineSettings *cls = context->cls;
   Settings *settings = context->settings;
-  if (cls->l_len <= maxfastload)
-    sr = context->fastload (name);
-  else
-    sr = (*(context->stationIndex()))[name];
+  sr = (*(context->stationIndex()))[name];
   if (sr) {
     Station *s = sr->load(context);
 
@@ -52,7 +49,7 @@ FILE *outfp, char mode, char format) {
     // This needs to be done here due to the timezones.
     Timestamp startt, endt;
     if (cls->b.isNull())
-      startt = Timestamp (time (NULL));
+      startt = Timestamp ((time_t)(time(NULL)));
     else {
       startt = Timestamp (cls->b, sr->timezone, context->settings);
       if (startt.isNull()) {
@@ -130,7 +127,17 @@ FILE *outfp, char mode, char format) {
         fmtbarf;
       {
         Dstr text_out;
-        context->rawMode (s, startt, endt, step, text_out);
+        context->rareModes (s, startt, endt, step, text_out, 0);
+        fprintf (outfp, "%s", text_out.aschar());
+      }
+      break;
+
+    case 'm':
+      if (format != 't')
+        fmtbarf;
+      {
+        Dstr text_out;
+        context->rareModes (s, startt, endt, step, text_out, 1);
         fprintf (outfp, "%s", text_out.aschar());
       }
       break;
@@ -214,7 +221,7 @@ loop_locations (TideContext *context, FILE *outfp) {
 
   if (mode != 'l') {
     if (!l)
-      cerr << "Warning:  No locations specified with -l; hence, no output." << endl;
+      fprintf (stderr, "Warning:  No locations specified with -l; hence, no output.\n");
     while (l) {
       if (is_first)
 	is_first = 0;
@@ -266,13 +273,16 @@ int main (int argc, char **argv)
 
   struct stat buf;
   if (stat (context->disabledisclaimerfile.aschar(), &buf)) {
-    cerr << "\
+    fprintf (stderr, "\
 -----------------------------------------------------------------------------\n\
              XTide   Copyright (C) 1998 David Flater.\n\
 \n\
 This software is provided under the terms of the GNU General Public\n\
 License, either version 2 of the License, or (at your option) any later\n\
 version.\n\
+\n\
+Although the package as a whole is GPL, some individual source files\n\
+are public domain.  Consult their header comments for details.\n\
 \n\
                         NOT FOR NAVIGATION\n\
 \n\
@@ -280,18 +290,18 @@ This program is distributed in the hope that it will be useful, but\n\
 WITHOUT ANY WARRANTY; without even the implied warranty of\n\
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  The author\n\
 assumes no liability for damages arising from use of this program OR\n\
-of the 'harmonics data' that is distributed with it.  For details, see\n\
+of any 'harmonics data' that might be distributed with it.  For details, see\n\
 the verbose documentation at http://www.flaterco.com/xtide/.\n\
 \n\
 This obnoxious message will go away permanently if you click \"Don't show\n\
 this again\" in the disclaimer window of the X windows client (xtide), or\n\
 if you create a file in your home directory called \".disableXTidedisclaimer\".\n\
 -----------------------------------------------------------------------------\n\
-\n";
+\n");
   }
 
   if (argc < 2 && context->cls->l_len == 0) {
-    cerr << "\
+    fprintf (stderr, "\
 Minimal usage:  tide -l \"Location name\" (or set the environment variable\n\
   XTIDE_DEFAULT_LOCATION to \"Location name\")\n\
 Other switches:\n\
@@ -302,9 +312,9 @@ Other switches:\n\
   -f h|p|t\n\
       Specify the output format as HTML, PNG, or text.  The default is text.\n\
       Only calendar and list modes can do HTML; only graph mode can do PNG.\n\
-  -m b|c|C|g|l|p|r|s\n\
-      Specify mode to be banner, calendar, alt. calendar, graph, list, plain,\n\
-      raw, or stats.  The default is plain.\n\
+  -m b|c|C|g|l|m|p|r|s\n\
+      Specify mode to be banner, calendar, alt. calendar, graph, list, medium\n\
+      rare, plain, raw, or stats.  The default is plain.\n\
   -o \"filename\"\n\
       Redirect output to the specified file (appends).\n\
   -s \"HH:MM\"\n\
@@ -315,7 +325,7 @@ Other switches:\n\
 \n\
 NOTE:  These are only the most important switches.  For information on\n\
 all of the switches, please read the verbose documentation at:\n\
-    http://www.flaterco.com/xtide/" << endl;
+    http://www.flaterco.com/xtide/\n");
     exit (-1);
   }
 
