@@ -1,4 +1,4 @@
-// 	$Id: xttpd.cc,v 1.10 2001/04/15 19:03:06 dave Exp $	
+// $Id: xttpd.cc,v 1.5 2003/02/20 15:19:56 flaterco Exp $
 /*  xttpd  XTide web server.
 
     Copyright (C) 1998  David Flater.
@@ -29,6 +29,13 @@ Dstr webmaster;
 ZoneIndex zi;
 int zoneinfo_doesnt_suck;
 
+// time control modes
+
+enum TC {	TC_ALL,		// year, month, day
+		TC_YEAR,	// year
+		TC_MONTH	// year, month
+};
+
 #include "purec++.hh"
 
 static void
@@ -287,11 +294,10 @@ list_location (Dstr &d, StationIndex *si, unsigned long i) {
   d += ".html\">";
   d += sr->name;
   d += "</a></td><td>";
-  {
-    Dstr fname;
-    sr->shortHarmonicsFileName (fname);
-    d += fname;
-  }
+  if (sr->is_reference_station())
+    d += "Ref";
+  else
+    d += "Sub";
   d += "</td><td>";
   Dstr tempc;
   sr->coordinates.print (tempc);
@@ -361,13 +367,12 @@ the tide data are updated.</p>");
 }
 
 static void
-add_time_control (Dstr &text, Dstr &url, unsigned y, unsigned m, unsigned d) {
-  text += "<h2> Time Control </h2>\n";
+add_time_control (enum TC type, Dstr &text, Dstr &url, unsigned y, unsigned m, unsigned d) {
   text += "<p> <form method=get action=\"";
   text += url;
   text += "\">\nYear: <select name=\"y\">\n";
   unsigned i;
-  for (i=globalfirstyear; i<=globallastyear; i++) {
+  for (i=dialogfirstyear; i<=dialoglastyear; i++) {
     if (i == y)
       text += "<option selected>";
     else
@@ -377,43 +382,46 @@ add_time_control (Dstr &text, Dstr &url, unsigned y, unsigned m, unsigned d) {
   }
   text += "</select>\n";
 
-  text += "Month: <select name=\"m\">\n";
-  for (i=1; i<=12; i++) {
-    if (i == m)
-      text += "<option selected>";
-    else
-      text += "<option>";
-    text += i;
-    text += "\n";
+  if(type != TC_YEAR) {
+    text += "Month: <select name=\"m\">\n";
+    for (i=1; i<=12; i++) {
+      if (i == m)
+        text += "<option selected>";
+      else
+        text += "<option>";
+      text += i;
+      text += "\n";
+    }
+    text += "</select>\n";
   }
-  text += "</select>\n";
 
-  text += "Day: <select name=\"d\">\n";
-  for (i=1; i<=31; i++) {
-    if (i == d)
-      text += "<option selected>";
-    else
-      text += "<option>";
-    text += i;
-    text += "\n";
+  if(type == TC_ALL)
+  {
+    text += "Day: <select name=\"d\">\n";
+    for (i=1; i<=31; i++) {
+      if (i == d)
+        text += "<option selected>";
+      else
+        text += "<option>";
+      text += i;
+      text += "\n";
+    }
+    text += "</select>\n";
   }
-  text += "</select>\n";
 
   text += "<input type=submit value=\"Go\"></form></p>";
-  text += "<p> Note:  predictions for some locations do not cover the\n\
-full span of years.&nbsp; If your browser returns a blank page or a \"no data\"\n\
-error, then the predictions that you requested are not available. </p>\n";
 }
 
 Timestamp
 parse_time_control (Dstr &filename, Dstr timezone, Settings *settings,
 Dstr &tc_out) {
   tc_out = (char *)NULL;
-  Timestamp ret (time(NULL));
+  Timestamp ret ((time_t)(time(NULL)));
   int i;
   if ((i = filename.strrchr ('?')) != -1) {
     unsigned y, m, d;
-    if ((sscanf (filename.ascharfrom(i), "?y=%u&m=%u&d=%u", &y, &m, &d)) == 3) {
+    m = d = 1;
+    if ((sscanf (filename.ascharfrom(i), "?y=%u&m=%u&d=%u", &y, &m, &d)) >= 1) {
       char temp[80];
       sprintf (temp, "%4u-%02u-%02u 00:00", y, m, d);
       Dstr in_iso (temp);
@@ -458,6 +466,10 @@ Content-type: text/html\n\
     top_p += "Local time:  ";
   else
     top_p += "Requested time:  ";
+#if __alpha
+  Dstr mungebuf;
+  start_tm.print (mungebuf, st->timeZone, context->settings);
+#endif
   Dstr nowbuf;
   start_tm.print (nowbuf, st->timeZone, context->settings);
   top_p += nowbuf;
@@ -483,21 +495,23 @@ for some locations. </p>\n";
   top_p += text_out;
   top_p += "</pre></p>\n";
 
-  add_time_control (top_p, myurl, y, m, d);
+  top_p += "<h2> Time Control </h2>\n";
+  add_time_control (TC_ALL, top_p, myurl, y, m, d);
+  top_p += "<p> Note:  predictions for some locations do not cover the\n\
+full span of years.&nbsp; If your browser returns a blank page or a \"no data\"\n\
+error, then the predictions that you requested are not available. </p>\n";
 
-  top_p += "<h2> Tide Calendars </h2>\n";
-  top_p += "<p> <a href=\"/calendar/month/";
-  top_p += i;
-  top_p += ".html";
-  top_p += tc_out;
-  top_p += "\"> One month</a> <br>\n";
-  top_p += "<a href=\"/calendar/year/";
-  top_p += i;
-  top_p += ".html";
-  top_p += tc_out;
-  top_p += "\"> One year</a> </p>\n";
-  top_p += "<p> Calendars will be generated for the month/year noted\n\
-at the top of this page. </p>\n";
+  top_p += "<h2> Monthly Tide Calendars </h2>\n";
+  Dstr monthurl ("/calendar/month/");
+  monthurl += i;
+  monthurl += ".html";
+  add_time_control (TC_MONTH, top_p, monthurl, y, m, d);
+
+  top_p += "<h2> Yearly Tide Calendars </h2>\n";
+  Dstr yearurl ("/calendar/year/");
+  yearurl += i;
+  yearurl += ".html";
+  add_time_control (TC_YEAR, top_p, yearurl, y, m, d);
 
   add_disclaimer (top_p);
   write (s, top_p.aschar(), top_p.length());
@@ -1003,9 +1017,18 @@ int main (int argc, char **argv)
   fd_set rdset;
   FD_ZERO (&rdset);
 
-  if (argc >= 2)
+  // The port number given to xttpd is not a "proper" command line
+  // setting recognized by XTide and doesn't deserve to be, so
+  // if we get one we hide it to keep CommandLineSettings from
+  // blowing up.
+
+  int portnum_evade = 0;
+  if (argc >= 2) {
     if (sscanf (argv[1], "%hu", &portnum) != 1)
       portnum = 80;
+    else
+      portnum_evade = 1;
+  }
 
   setup_socket (portnum, listen_socket);
   dont_be_root();
@@ -1015,18 +1038,23 @@ int main (int argc, char **argv)
   {
     ConfigDefaults cd;
     UserDefaults ud;
-    CommandLineSettings cls (argc, argv);
+    CommandLineSettings cls (
+      portnum_evade ? argc-1 : argc,
+      portnum_evade ? &(argv[1]) : argv);
     settings.supersedeBy (cd);
     settings.supersedeBy (ud);
     settings.supersedeBy (cls);
   }
 
-  context = new TideContext (argc, argv, new Colors (settings), &settings);
+  context = new TideContext (
+      portnum_evade ? argc-1 : argc,
+      portnum_evade ? &(argv[1]) : argv,
+      new Colors (settings), &settings);
   // Build the indices.
   zi.add (context->stationIndex());
   // Avoid issuing the zoneinfo warning a million times.
   {
-    Timestamp nowarn(time(NULL));
+    Timestamp nowarn((time_t)(time(NULL)));
     zoneinfo_doesnt_suck = nowarn.zoneinfo_doesnt_suck(&settings);
   }
   log ("Accepting connections.", LOG_NOTICE);
