@@ -1,4 +1,4 @@
-// $Id: xxApplicationDefaults.cc,v 1.1 2000/08/30 22:59:59 dave Exp $
+// $Id: xxApplicationDefaults.cc,v 1.2 2002/10/01 18:04:31 flaterco Exp $
 /*  xxApplicationDefaults  Settings from XResources.
 
     Copyright (C) 1998  David Flater.
@@ -50,6 +50,8 @@ xxApplicationDefaults::xxApplicationDefaults (XrmDatabase database) {
     z = (char *)xv.addr;
   if (XrmGetResource (database, "xtide*extralines", "XTide*extralines", str_type, &xv))
     el = (char *)xv.addr;
+  if (XrmGetResource (database, "xtide*flatearth", "XTide*flatearth", str_type, &xv))
+    fe = (char *)xv.addr;
   if (XrmGetResource (database, "xtide*datefmt", "XTide*datefmt", str_type, &xv))
     df = (char *)xv.addr;
   if (XrmGetResource (database, "xtide*hourfmt", "XTide*hourfmt", str_type, &xv))
@@ -62,6 +64,8 @@ xxApplicationDefaults::xxApplicationDefaults (XrmDatabase database) {
     nf = (char *)xv.addr;
   if (XrmGetResource (database, "xtide*nosunmoon", "XTide*nosunmoon", str_type, &xv))
     ns = (char *)xv.addr;
+  if (XrmGetResource (database, "xtide*infer", "XTide*infer", str_type, &xv))
+    in = (char *)xv.addr;
 
   if (XrmGetResource (database, "xtide*gaspect", "XTide*gaspect", str_type, &xv)) {
     ga_isnull = 0;
