@@ -1,5 +1,5 @@
+// $Id: xxContext.cc,v 1.3 2002/11/20 17:43:25 flaterco Exp $
 /*  xxContext    X-Toolkit-on-a-steek.  Packaging of assorted XCrap.
-    Last modified 1998-05-28
 
     Copyright (C) 1998  David Flater.
 
@@ -135,7 +135,7 @@ xxContext::xxContext (int &argc, char **argv, Colors *&colors, Settings *&settin
     {
       for (int a=0; a<argc; a++)
         if (!strncmp (argv[a], "-ge", 3))
-          strcpy (argv[a], "");
+          strcpy (argv[a], "-XX");
     }
 
     display = XtOpenDisplay (context, NULL, "XTide", "XTide",
@@ -443,7 +443,7 @@ mpcolormap, unsigned mpdepth) {
     data, &temp, NULL, &xpma);
   switch (retval) {
   case XpmColorError:
-    cerr << "Warning:  Non-fatal Xpm color error reported." << endl;
+    fprintf (stderr, "Warning:  Non-fatal Xpm color error reported.\n");
   case XpmSuccess:
     return temp;
   case XpmOpenFailed:
@@ -464,7 +464,7 @@ mpcolormap, unsigned mpdepth) {
     details = "Fatal Xpm error of unrecognized type reported.";
     barf (XPM_ERROR, details);
   }
-  cerr << "Warning:  Non-fatal Xpm error of unrecognized type reported." << endl;
+  fprintf (stderr, "Warning:  Non-fatal Xpm error of unrecognized type reported.\n");
   return temp;
 }
 
@@ -792,12 +792,9 @@ unsigned short blue, Visual *gcvisual, Colormap gccolormap, int sucks) {
         closest = tryit;
       }
     }
-    cerr << "XTide:  Can't allocate color; using substitute (badness " <<
-    (unsigned long)(sqrt (closest_distance)) << ")" << endl;
+    fprintf (stderr, "XTide:  Can't allocate color; using substitute (badness %lu)\n", (unsigned long)(sqrt (closest_distance)));
     if (!(XAllocColor (display, gccolormap, &closest)))
-      cerr <<
-      "XTide:  ACK!  Can't allocate that either!  Expect color shifting." <<
-      endl;
+      fprintf (stderr, "XTide:  ACK!  Can't allocate that either!  Expect color shifting.\n");
     return (closest.pixel);
   }
   return (cdef.pixel);
