@@ -1,4 +1,4 @@
-// $Id: xxTitleScreen.cc,v 1.1 2001/03/23 00:59:00 dave Exp $
+// $Id: xxTitleScreen.cc,v 1.2 2002/10/04 13:44:01 flaterco Exp $
 /*  xxTitleScreen  XTide title screen.
 
     Copyright (C) 1998  David Flater.
@@ -68,7 +68,7 @@ xxWindow (in_xtidecontext, context, 0, in_grabkind) {
 // The PNG title screen has too much overhead.
 //   if (mypopup->display_sucks) {
     if (!(titlescreenpixmap = mypopup->makePixmap (titlescreen_xpm))) {
-      cerr << "XTide Warning: can't create pixmap for title screen (can't get enough colors)." << endl;
+      fprintf (stderr, "XTide Warning: can't create pixmap for title screen (can't get enough colors).\n");
       // Substitute a blank pixmap
       titlescreenpixmap = mypopup->makePixmap (titlescreenwidth, titlescreenheight);
     }
