@@ -1,4 +1,4 @@
-// $Id: xxXTideRoot.cc,v 1.1 2000/08/30 23:00:16 dave Exp $
+// $Id: xxXTideRoot.cc,v 1.7 2003/02/04 21:26:48 flaterco Exp $
 /*  xxXTideRoot  XTide "root" window
 
     Copyright (C) 1998  David Flater.
@@ -127,6 +127,18 @@ in your date/time formats."); \
   default:
     assert (0);
   }
+  switch (fechoice->choice()) {
+  case 0:
+    newset.fe = "y";
+    break;
+  case 1:
+    newset.fe = "n";
+    break;
+  case 2:
+    break;
+  default:
+    assert (0);
+  }
   switch (nfchoice->choice()) {
   case 0:
     newset.nf = "y";
@@ -151,6 +163,18 @@ in your date/time formats."); \
   default:
     assert (0);
   }
+  switch (inchoice->choice()) {
+  case 0:
+    newset.in = "y";
+    break;
+  case 1:
+    newset.in = "n";
+    break;
+  case 2:
+    break;
+  default:
+    assert (0);
+  }
   switch (tlchoice->choice()) {
   case 0:
     newset.tl = "y";
@@ -339,26 +363,43 @@ void RootCloseHandler (Widget w, XtPointer client_data,
 // This is called by the constructor and also by hide()
 void
 xxXTideRoot::addTwoButtons() {
-  Arg buttonargs[4] =  {
-    {XtNvisual, (XtArgVal)mycontext->visual},
-    {XtNcolormap, (XtArgVal)mycontext->colormap},
-    {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
-    {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]}
-  };
-
   {
+    Arg buttonargs[10] =  {
+      {XtNvisual, (XtArgVal)mycontext->visual},
+      {XtNcolormap, (XtArgVal)mycontext->colormap},
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNfromVert, (XtArgVal)viewport->manager},
+      {XtNfromHoriz, (XtArgVal)savebutton->manager},
+      {XtNleft, (XtArgVal)XawChainLeft},
+      {XtNright, (XtArgVal)XawChainLeft},
+      {XtNtop, (XtArgVal)XawChainBottom},
+      {XtNbottom, (XtArgVal)XawChainBottom}
+    };
     Widget buttonwidget = XtCreateManagedWidget ("Dismiss", commandWidgetClass,
-      box->manager, buttonargs, 4);
+      form->manager, buttonargs, 10);
     XtAddCallback (buttonwidget, XtNcallback, dismissCallback,
      (XtPointer)this);
-    dismissbutton = new xxContext (box, buttonwidget);
+    dismissbutton = new xxContext (form, buttonwidget);
   }
   {
+    Arg buttonargs[10] =  {
+      {XtNvisual, (XtArgVal)mycontext->visual},
+      {XtNcolormap, (XtArgVal)mycontext->colormap},
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNfromVert, (XtArgVal)viewport->manager},
+      {XtNfromHoriz, (XtArgVal)dismissbutton->manager},
+      {XtNleft, (XtArgVal)XawChainLeft},
+      {XtNright, (XtArgVal)XawChainLeft},
+      {XtNtop, (XtArgVal)XawChainBottom},
+      {XtNbottom, (XtArgVal)XawChainBottom}
+    };
     Widget buttonwidget = XtCreateManagedWidget ("?", commandWidgetClass,
-      box->manager, buttonargs, 4);
+      form->manager, buttonargs, 10);
     XtAddCallback (buttonwidget, XtNcallback, helpCallback,
       (XtPointer)this);
-    helpbutton = new xxContext (box, buttonwidget);
+    helpbutton = new xxContext (form, buttonwidget);
   }
 }
 
@@ -382,21 +423,65 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
     RootCloseHandler, (XtPointer)this);
 
   {
-    Arg boxargs[3] =  {
+    Arg formargs[3] =  {
       {XtNbackground, (XtArgVal)mycontext->pixels[Colors::background]},
       {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
       {XtNvSpace, (XtArgVal)0}
     };
-    Widget boxwidget = XtCreateManagedWidget ("", boxWidgetClass,
-      mycontext->manager, boxargs, 3);
-    box = new xxContext (mycontext, boxwidget);
+    Widget formwidget = XtCreateManagedWidget ("", formWidgetClass,
+      mycontext->manager, formargs, 3);
+    form = new xxContext (mycontext, formwidget);
   }
 
   {
+    Arg labelargs[6] =  {
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::background]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNleft, (XtArgVal)XawChainLeft},
+      {XtNright, (XtArgVal)XawChainLeft},
+      {XtNtop, (XtArgVal)XawChainTop},
+      {XtNbottom, (XtArgVal)XawChainTop}
+    };
     Widget labelwidget = XtCreateManagedWidget (
       "-------------- XTide Control Panel --------------",
-      labelWidgetClass, box->manager, args, 2);
-    label = new xxContext (box, labelwidget);
+      labelWidgetClass, form->manager, labelargs, 6);
+    label = new xxContext (form, labelwidget);
+  }
+
+  Arg vpargs[9] =  {
+    {XtNbackground, (XtArgVal)mycontext->pixels[Colors::background]},
+    {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+    {XtNallowVert, (XtArgVal)1},
+    {XtNforceBars, (XtArgVal)1},
+    {XtNfromVert, (XtArgVal)label->manager},
+    {XtNleft, (XtArgVal)XawChainLeft},
+    {XtNright, (XtArgVal)XawChainRight},
+    {XtNtop, (XtArgVal)XawChainTop},
+    {XtNbottom, (XtArgVal)XawChainBottom}
+  };
+
+  Widget viewportwidget = XtCreateManagedWidget ("", viewportWidgetClass,
+    form->manager, vpargs, 9);
+  viewport = new xxContext (form, viewportwidget);
+
+  Widget scrollbarwidget;
+  assert (scrollbarwidget = XtNameToWidget (viewportwidget, "vertical"));
+  Arg sbargs[2] = {
+    {XtNbackground, (XtArgVal)mycontext->pixels[Colors::background]},
+    {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]}
+  };
+  XtSetValues (scrollbarwidget, sbargs, 2);
+
+  {
+    Arg boxargs[4] =  {
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::background]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNvSpace, (XtArgVal)0},
+      {XtNorientation, (XtArgVal)XtorientVertical}
+    };
+    Widget boxwidget = XtCreateManagedWidget ("", boxWidgetClass,
+      viewport->manager, boxargs, 4);
+    viewbox = new xxContext (viewport, boxwidget);
   }
 
   // Get current settings.  (Another duplication...)
@@ -405,16 +490,9 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
   // Add dialogs for all of the colors
   int a;
   for (a=0; a<numcolors; a++)
-    colordiags[a] = new xxHorizDialog (box, colordesc[a],
+    colordiags[a] = new xxHorizDialog (viewbox, colordesc[a],
       ud->colors[a].aschar());
 
-  Arg buttonargs[4] =  {
-    {XtNvisual, (XtArgVal)mycontext->visual},
-    {XtNcolormap, (XtArgVal)mycontext->colormap},
-    {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
-    {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]}
-  };
-
   // Add toggles
   static char *togglechoices[] = {"Yes", "No", "(blank)", NULL};
   {
@@ -426,8 +504,20 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 0;
-    elchoice = new xxMultiChoice (box,
-      "Draw datum and MSL lines in tide graphs?", togglechoices, firstchoice);
+    elchoice = new xxMultiChoice (viewbox,
+      "Draw datum and Mean Tide Level lines in tide graphs?", togglechoices, firstchoice);
+  }
+  {
+    Dstr t = ud->fe;
+    unsigned firstchoice;
+    if (t.isNull())
+      firstchoice = 2;
+    else if (t == "n")
+      firstchoice = 1;
+    else
+      firstchoice = 0;
+    fechoice = new xxMultiChoice (viewbox,
+      "Prefer flat map to round globe location chooser?", togglechoices, firstchoice);
   }
   {
     Dstr t = ud->nf;
@@ -438,7 +528,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 0;
-    nfchoice = new xxMultiChoice (box,
+    nfchoice = new xxMultiChoice (viewbox,
       "Draw tide graphs as line graphs?", togglechoices, firstchoice);
   }
   {
@@ -450,8 +540,20 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 0;
-    nschoice = new xxMultiChoice (box,
-      "Suppress sunrise, sunset, moon phases?", togglechoices, firstchoice);
+    nschoice = new xxMultiChoice (viewbox,
+      "Suppress sun and moon events in output?", togglechoices, firstchoice);
+  }
+  {
+    Dstr t = ud->in;
+    unsigned firstchoice;
+    if (t.isNull())
+      firstchoice = 2;
+    else if (t == "n")
+      firstchoice = 1;
+    else
+      firstchoice = 0;
+    inchoice = new xxMultiChoice (viewbox,
+      "Use inferred constituents (expert only)?", togglechoices, firstchoice);
   }
   {
     Dstr t = ud->tl;
@@ -462,7 +564,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 0;
-    tlchoice = new xxMultiChoice (box,
+    tlchoice = new xxMultiChoice (viewbox,
       "Draw depth lines on top of tide graph?", togglechoices, firstchoice);
   }
   {
@@ -474,7 +576,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 0;
-    zchoice = new xxMultiChoice (box,
+    zchoice = new xxMultiChoice (viewbox,
       "Coerce all time zones to UTC?", togglechoices, firstchoice);
   }
 
@@ -491,7 +593,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       firstchoice = 1;
     else
       firstchoice = 2;
-    uchoice = new xxMultiChoice (box,
+    uchoice = new xxMultiChoice (viewbox,
       "Preferred units of length:", unitschoices, firstchoice);
   }
 
@@ -506,31 +608,31 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       else
         firstchoice = (unsigned) ((ud->gl + 180.0) / 30.0);
     }
-    glchoice = new xxMultiChoice (box,
+    glchoice = new xxMultiChoice (viewbox,
       "Default center longitude for globe:", glchoices, firstchoice);
   }
 
   // Date/time format dialogs
-  dfdiag = new xxHorizDialog (box,
+  dfdiag = new xxHorizDialog (viewbox,
     "Strftime style format string for printing dates.", ud->df.aschar());
-  hfdiag = new xxHorizDialog (box,
+  hfdiag = new xxHorizDialog (viewbox,
     "Strftime style format string for printing hour labels on time axis.",
     ud->hf.aschar());
-  tfdiag = new xxHorizDialog (box,
+  tfdiag = new xxHorizDialog (viewbox,
     "Strftime style format string for printing times.", ud->tf.aschar());
 
   // Unsigned numbers.
-  gwchoice = new xxUnsignedChooser (box,
+  gwchoice = new xxUnsignedChooser (viewbox,
     "Default width for tide graphs (pixels):", ud->gw, ud->gw_isnull, mingwidth);
-  ghchoice = new xxUnsignedChooser (box,
+  ghchoice = new xxUnsignedChooser (viewbox,
     "Default height for tide graphs and clocks (pixels):", ud->gh, ud->gh_isnull,
     mingheight);
-  cwchoice = new xxUnsignedChooser (box,
+  cwchoice = new xxUnsignedChooser (viewbox,
     "Default width for tide clocks (pixels):", ud->cw, ud->cw_isnull, mingwidth);
-  twchoice = new xxUnsignedChooser (box,
+  twchoice = new xxUnsignedChooser (viewbox,
     "Default width of ASCII graphs and banners (characters):", ud->tw,
      ud->tw_isnull, minttywidth);
-  thchoice = new xxUnsignedChooser (box,
+  thchoice = new xxUnsignedChooser (viewbox,
     "Default height of ASCII graphs (characters):", ud->th, ud->th_isnull,
     minttyheight);
 
@@ -541,7 +643,7 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       strcpy (temp, "");
     else
       sprintf (temp, "%f", ud->ga);
-    gadiag = new xxHorizDialog (box,
+    gadiag = new xxHorizDialog (viewbox,
       "Default aspect for tide graphs.", temp);
   }
   {
@@ -550,23 +652,46 @@ xxXTideRoot::xxXTideRoot (int in_argc, char **in_argv) {
       strcpy (temp, "");
     else
       sprintf (temp, "%f", ud->lw);
-    lwdiag = new xxHorizDialog (box,
+    lwdiag = new xxHorizDialog (viewbox,
       "Width for lines in line graphs (pixels, pos. real number).", temp);
   }
 
   {
+    Arg applyargs[9] =  {
+      {XtNvisual, (XtArgVal)mycontext->visual},
+      {XtNcolormap, (XtArgVal)mycontext->colormap},
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNfromVert, (XtArgVal)viewport->manager},
+      {XtNleft, (XtArgVal)XawChainLeft},
+      {XtNright, (XtArgVal)XawChainLeft},
+      {XtNtop, (XtArgVal)XawChainBottom},
+      {XtNbottom, (XtArgVal)XawChainBottom}
+    };
     Widget buttonwidget = XtCreateManagedWidget ("Apply", commandWidgetClass,
-      box->manager, buttonargs, 4);
+      form->manager, applyargs, 9);
     XtAddCallback (buttonwidget, XtNcallback, applyCallback,
      (XtPointer)this);
-    applybutton = new xxContext (box, buttonwidget);
+    applybutton = new xxContext (form, buttonwidget);
   }
   {
+    Arg saveargs[10] =  {
+      {XtNvisual, (XtArgVal)mycontext->visual},
+      {XtNcolormap, (XtArgVal)mycontext->colormap},
+      {XtNbackground, (XtArgVal)mycontext->pixels[Colors::button]},
+      {XtNforeground, (XtArgVal)mycontext->pixels[Colors::foreground]},
+      {XtNfromVert, (XtArgVal)viewport->manager},
+      {XtNfromHoriz, (XtArgVal)applybutton->manager},
+      {XtNleft, (XtArgVal)XawChainLeft},
+      {XtNright, (XtArgVal)XawChainLeft},
+      {XtNtop, (XtArgVal)XawChainBottom},
+      {XtNbottom, (XtArgVal)XawChainBottom}
+    };
     Widget buttonwidget = XtCreateManagedWidget ("Save", commandWidgetClass,
-      box->manager, buttonargs, 4);
+      form->manager, saveargs, 10);
     XtAddCallback (buttonwidget, XtNcallback, saveCallback,
      (XtPointer)this);
-    savebutton = new xxContext (box, buttonwidget);
+    savebutton = new xxContext (form, buttonwidget);
   }
 
   addTwoButtons();
@@ -626,8 +751,10 @@ void xxXTideRoot::global_redraw() {
     colordiags[a]->global_redraw();
 
   elchoice->global_redraw();
+  fechoice->global_redraw();
   nfchoice->global_redraw();
   nschoice->global_redraw();
+  inchoice->global_redraw();
   tlchoice->global_redraw();
   zchoice->global_redraw();
   uchoice->global_redraw();
@@ -664,8 +791,12 @@ void xxXTideRoot::global_redraw() {
   };
   if (label)
     XtSetValues (label->manager, args, 2);
-  if (box)
-    XtSetValues (box->manager, args, 2);
+  if (viewbox)
+    XtSetValues (viewbox->manager, args, 2);
+  if (viewport)
+    XtSetValues (viewport->manager, args, 2);
+  if (form)
+    XtSetValues (form->manager, args, 2);
   if (mycontext)
     XtSetValues (mycontext->manager, args, 2);
 }
@@ -692,12 +823,16 @@ xxXTideRoot::~xxXTideRoot () {
   delete thchoice;
   delete cwchoice;
   delete elchoice;
+  delete fechoice;
   delete nfchoice;
   delete nschoice;
+  delete inchoice;
   delete tlchoice;
   delete zchoice;
   delete uchoice;
-  delete box;
+  delete viewbox;
+  delete viewport;
+  delete form;
   delete mycontext;
   delete myTideContext;
   delete ud;
@@ -727,12 +862,20 @@ void xxXTideRoot::newGraph (Station *s, Timestamp t) {
 }
 
 void xxXTideRoot::newGraph (StationRef *sr) {
-  Timestamp t (time(NULL));
+  Timestamp t ((time_t)(time(NULL)));
   newGraph (sr->load (myTideContext), t);
 }
 
 void xxXTideRoot::newText (Station *s, Timestamp t) {
-  (void) new xxTextMode (myTideContext, mycontext, s, t);
+  (void) new xxTextMode (myTideContext, mycontext, s, 2, t);
+}
+
+void xxXTideRoot::newRaw (Station *s, Timestamp t) {
+  (void) new xxTextMode (myTideContext, mycontext, s, 0, t);
+}
+
+void xxXTideRoot::newMediumRare (Station *s, Timestamp t) {
+  (void) new xxTextMode (myTideContext, mycontext, s, 1, t);
 }
 
 void xxXTideRoot::newClock (Station *s, int nobuttonsflag) {
@@ -740,15 +883,12 @@ void xxXTideRoot::newClock (Station *s, int nobuttonsflag) {
 }
 
 void xxXTideRoot::newWindows (CommandLineSettings *cls) {
-  Timestamp t (time(NULL));
+  Timestamp t ((time_t)(time(NULL)));
   assert (cls);
   struct CommandLineSettings::lnode *l = cls->l;
   while (l) {
     StationRef *sr;
-    if (cls->l_len <= maxfastload)
-      sr = myTideContext->fastload (l->name);
-    else
-      sr = (*(myTideContext->stationIndex()))[l->name];
+    sr = (*(myTideContext->stationIndex()))[l->name];
     if (sr) {
       if (!(cls->b.isNull())) {
         t = Timestamp (cls->b, sr->timezone, myTideContext->settings);
@@ -780,6 +920,20 @@ void xxXTideRoot::newWindows (CommandLineSettings *cls) {
 }
 
 void xxXTideRoot::newChooser (XtGrabKind in_grabkind) {
+  if (myTideContext->settings->fe == "n")
+    (void) new xxGlobe (mycontext, *(myTideContext->stationIndex(in_grabkind)),
+      myTideContext);
+  else
+    (void) new xxMap (mycontext, *(myTideContext->stationIndex(in_grabkind)),
+      myTideContext);
+}
+
+void xxXTideRoot::newMap (XtGrabKind in_grabkind) {
+  (void) new xxMap (mycontext, *(myTideContext->stationIndex(in_grabkind)),
+    myTideContext);
+}
+
+void xxXTideRoot::newGlobe (XtGrabKind in_grabkind) {
   (void) new xxGlobe (mycontext, *(myTideContext->stationIndex(in_grabkind)),
     myTideContext);
 }
